package envoy.client.util;

import java.io.File;
import java.io.IOException;
import java.util.logging.*;

import envoy.client.Config;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>EnvoyLogger.java</strong><br>
 * Created: <strong>14 Dec 2019</strong><br>
 *
 * @author Leon Hofmeister
 * @author Kai S. K. Engelbart
 * @since Envoy v0.2-alpha
 */
public class EnvoyLog {

	private static FileHandler		fileHandler;
	private static StreamHandler	consoleHandler;

	static {
		// Remove default console handler
		LogManager.getLogManager().reset();

		// Configure log file
		File logFile = new File(Config.getInstance().getHomeDirectory(), "log/envoy_user.log");
		logFile.getParentFile().mkdirs();

		// Configure formatting
		System.setProperty("java.util.logging.SimpleFormatter.format", "[%1$tF %1$tT] [%4$-7s] [%3$s] %5$s %n");
		SimpleFormatter formatter = new SimpleFormatter();

		try {
			fileHandler = new FileHandler(logFile.getAbsolutePath());
			fileHandler.setLevel(Config.getInstance().getFileLevelBarrier());
			fileHandler.setFormatter(formatter);
		} catch (SecurityException | IOException e) {
			e.printStackTrace();
		}
		consoleHandler = new StreamHandler(System.out, formatter);
		consoleHandler.setLevel(Config.getInstance().getConsoleLevelBarrier());
		consoleHandler.setFormatter(formatter);
	}

	private EnvoyLog() {}

	/**
	 * Creates a {@link Logger} with a specified name
	 *
	 * @param name the name of the {@link Logger} to create
	 * @return the created {@link Logger}
	 * @since Envoy v0.2-alpha
	 */
	public static Logger getLogger(String name) {
		// Get a logger with the specified name
		Logger logger = Logger.getLogger(name);

		// Add handlers
		if (fileHandler != null) logger.addHandler(fileHandler);
		if (consoleHandler != null) logger.addHandler(consoleHandler);

		return logger;
	}

	/**
	 * @param fileLevelBarrier the severity below which logRecords will not be
	 *                         written to the log file. At or above they'll also be
	 *                         logged in a file. Can be written either in Digits
	 *                         from 0 - 1000 or with the according name of the level
	 * @since Envoy v0.2-alpha
	 */
	public static void setFileLevelBarrier(Level fileLevelBarrier) { if (fileHandler != null) fileHandler.setLevel(fileLevelBarrier); }

	/**
	 * @param consoleLevelBarrier the severity below which logRecords will not be
	 *                            written to the console. At or above they'll also
	 *                            be logged in a file. Can be written either in
	 *                            digits from 0 - 1000 or with the according name of
	 *                            the level
	 * @since Envoy v0.2-alpha
	 */
	public static void setConsoleLevelBarrier(Level consoleLevelBarrier) {
		if (consoleHandler != null) consoleHandler.setLevel(consoleLevelBarrier);
	}
}
